@@ -7,6 +7,8 @@
 
 #include "util.h"
 
+static struct tm tmnow;
+static time_t now;
 static char *line;
 static size_t linesize;
 
@@ -44,14 +46,7 @@ printfeed(FILE *fp, const char *feedname)
 	while ((linelen = getline(&line, &linesize, fp)) > 0) {
 		if (line[linelen - 1] == '\n')
 			line[--linelen] = '\0';
-		if (!parseline(line, fields))
-			break;
-
-		parsedtime = 0;
-		if (strtotime(fields[FieldUnixTimestamp], &parsedtime))
-			continue;
-		if (!(tm = localtime(&parsedtime)))
-			err(1, "localtime");
+		parseline(line, fields);
 
 		fputs("<entry>\n\t<title>", stdout);
 		if (feedname[0]) {
@@ -66,14 +61,27 @@ printfeed(FILE *fp, const char *feedname)
 			xmlencode(fields[FieldLink], stdout);
 			fputs("\" />\n", stdout);
 		}
+		/* prefer link over id for Atom <id>. */
+		fputs("\t<id>", stdout);
+		if (fields[FieldLink][0])
+			xmlencode(fields[FieldLink], stdout);
+		else if (fields[FieldId][0])
+			xmlencode(fields[FieldId], stdout);
+		fputs("</id>\n", stdout);
 		if (fields[FieldEnclosure][0]) {
 			fputs("\t<link rel=\"enclosure\" href=\"", stdout);
 			xmlencode(fields[FieldEnclosure], stdout);
 			fputs("\" />\n", stdout);
 		}
-		fprintf(stdout, "\t<published>%04d-%02d-%02dT%02d:%02d:%02dZ</published>\n",
+
+		parsedtime = 0;
+		if (strtotime(fields[FieldUnixTimestamp], &parsedtime) ||
+		    !(tm = gmtime(&parsedtime)))
+			tm = &tmnow;
+		fprintf(stdout, "\t<updated>%04d-%02d-%02dT%02d:%02d:%02dZ</updated>\n",
 		        tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
 		        tm->tm_hour, tm->tm_min, tm->tm_sec);
+
 		if (fields[FieldAuthor][0]) {
 			fputs("\t<author><name>", stdout);
 			xmlencode(fields[FieldAuthor], stdout);
@@ -98,6 +106,7 @@ printfeed(FILE *fp, const char *feedname)
 int
 main(int argc, char *argv[])
 {
+	struct tm *tm;
 	FILE *fp;
 	char *name;
 	int i;
@@ -110,9 +119,20 @@ main(int argc, char *argv[])
 			err(1, "pledge");
 	}
 
+	if ((now = time(NULL)) == -1)
+		err(1, "time");
+	if (!(tm = gmtime_r(&now, &tmnow)))
+		err(1, "gmtime_r");
+
 	fputs("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
-	      "<feed xmlns=\"http://www.w3.org/2005/Atom\" xml:lang=\"en\">\n",
-	      stdout);
+	      "<feed xmlns=\"http://www.w3.org/2005/Atom\">\n"
+	      "\t<title type=\"text\">Newsfeed</title>\n"
+	      "\t<author><name>sfeed</name></author>\n", stdout);
+	printf("\t<id>urn:newsfeed:%lld</id>\n"
+	       "\t<updated>%04d-%02d-%02dT%02d:%02d:%02dZ</updated>\n",
+	       (long long)now,
+	       tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
+	       tm->tm_hour, tm->tm_min, tm->tm_sec);
 
 	if (argc == 1) {
 		printfeed(stdin, "");
