@@ -23,23 +23,22 @@ printfeed(FILE *fp, const char *feedname)
 	while ((linelen = getline(&line, &linesize, fp)) > 0) {
 		if (line[linelen - 1] == '\n')
 			line[--linelen] = '\0';
-		if (!parseline(line, fields))
-			break;
+		parseline(line, fields);
 
 		parsedtime = 0;
-		if (strtotime(fields[FieldUnixTimestamp], &parsedtime))
-			continue;
-		if (!(tm = localtime(&parsedtime)))
-			err(1, "localtime");
+		if (!strtotime(fields[FieldUnixTimestamp], &parsedtime) &&
+		    (tm = localtime(&parsedtime))) {
+			if (parsedtime >= comparetime)
+				fputs("N ", stdout);
+			else
+				fputs("  ", stdout);
+			fprintf(stdout, "%04d-%02d-%02d %02d:%02d  ",
+			        tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
+			        tm->tm_hour, tm->tm_min);
+		} else {
+			fputs("                    ", stdout);
+		}
 
-		if (parsedtime >= comparetime)
-			fputs("N ", stdout);
-		else
-			fputs("  ", stdout);
-
-		fprintf(stdout, "%04d-%02d-%02d %02d:%02d  ",
-		        tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
-		        tm->tm_hour, tm->tm_min);
 		if (feedname[0]) {
 			printutf8pad(stdout, feedname, 15, ' ');
 			fputs("  ", stdout);
